// Copyright (C) 1999-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
// As a special exception to the GNU General Public License, permission is 
// granted for additional uses of the text contained in its release 
// of ccRTP.
// 
// The exception is that, if you link the ccRTP library with other
// files to produce an executable, this does not by itself cause the
// resulting executable to be covered by the GNU General Public License.
// Your use of that executable is in no way restricted on account of
// linking the ccRTP library code into it.
// 
// This exception does not however invalidate any other reasons why
// the executable file might be covered by the GNU General Public License.
// 
// This exception applies only to the code released under the 
// name ccRTP.  If you copy code from other releases into a copy of
// ccRTP, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
// 
// If you write modifications of your own for ccRTP, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.  

/** 
 * @file private.h 
 * @brief Declaration of ccRTP internal stuff.
 *
 * This header includes the autogenerated config.h, the general header 
 * (rtp.h) and the nana assertion checking library headers.
 */

#include "config.h"
#undef	PACKAGE
#undef	VERSION
#include <cc++/config.h>
#include <cc++/socket.h>
#include <cc++/export.h>
#include "rtp.h"
#include "rtpext.h"

// This is only of interest for developers, so it is included only if
// --with-nana flag is given to configure
#ifdef HAVE_LIBNANA
#   include "nana.h"
#   include "DL.h"
#else
#   define I(e)    /* empty */
#   define DL(e)   /* empty */
#   define VDL(e)  /* empty */
#endif

#ifdef   HAVE_CSTDLIB
#include <cstdlib>
#else
#   ifdef   HAVE_STDLIB_H
#      include <stdlib.h>
#   else
#      warning "Your system does not have cstdlib nor stdlib.h"
#define getenv(w) ("unidentifiable_user")
#   endif //HAVE_STDLIB_H
#endif //HAVE_CSTDLIB

#ifdef HAVE_SYS_UTSNAME_H
#   include <sys/utsname.h>
#else
#   error "Portability problem: you do not have sys/utsname.h"
#endif

#ifdef	__NAMESPACES__
namespace ost {
#endif


// This is for the main page in Doxygen documentation
/**
 * @mainpage ccRTP Library Reference
 * @section overview Overview (README)
 * @verbinclude README
 * @section todolist TODO List
 * @verbinclude TODO
 */

// This will make doxygen to generate example descriptions
/**
 * @example rtpduphello.cpp
 */
/** 
 * @example rtphello.cpp
 */
/**
 * @example audio.h
 */
/**
 * @example audiotx.cpp
 */
/**
 * @example audiorx.cpp
 */

#pragma pack(1)
struct ReceiverInfo
{
	uint8 flost;
	uint8 lost_msb;
	uint16 lost_lsb;
	uint32 highest_seqnum;
	uint32 jitter;
	uint32 lsr;
	uint32 dlsr;
};

/**
 * @typedef RRBlock
 */
typedef struct 
{
	uint32 ssrc;
	ReceiverInfo rinfo;
}       RRBlock;

/**
 * @struct SenderInfo
 */
struct SenderInfo
{
	uint32 NTP_msb;
	uint32 NTP_lsb;
	uint32 RTPtimestamp;
	uint32 packet_count;
	uint32 octet_count;
};

/**
 *
 */
typedef struct 
{
	uint32 ssrc;
	SenderInfo sinfo;
	RRBlock blocks [0];
}       SendReport;

/**
 * @typedef RecvReport
 *
 * @short raw structure of every receiver report in an SR or RR RTCP
 *        packet. 
 */
typedef struct 
{
	uint32 ssrc;
	RRBlock blocks [0];
}       RecvReport;

/**
 *
 */
typedef struct 
{
	uint8 type;
	uint8 len;
}       SDESItem;

typedef struct 
{
	uint32 ssrc;
	SDESItem item;
}       SDESChunk;

typedef struct 
{
	uint32 ssrc;          ///< source leaving
	uint8 length;         ///< [optional] length of reason
}       BYEPacket;

typedef struct 
{
	char name [4];
	char* data;
}       APPPacket;

/**
 * @struct RTCPPacket
 */
struct RTCPPacket
{
	RTCPFixedHeader fh;
	union 
	{
		SendReport SR;
		RecvReport RR;
		SDESChunk SDES; 
		BYEPacket BYE;
		APPPacket APP;
	}       info;
};

/**
 *
 */
typedef struct {
	RTCPFixedHeader fh;
	uint32 ssrc;          ///< SSRC identifier of sender
	uint8 length;         ///< [optional] length of reason
	char const reason[0];  ///< [optional] reason text (not null terminated)
}       BYEHeader;

typedef struct {
	RTCPFixedHeader fh;
	uint32 ssrc;
	char name[4];
}       RTCPAPPHeader;
#pragma pack()

#ifdef	__NAMESPACES__
};
#endif

/** EMACS **
 * Local variables:
 * mode: c++
 * c-basic-offset: 8
 * End:
 */
